@@ -10,20 +10,11 @@ class DBPacemaker:
     Based on flask-sqlalchemy and Flask-APScheduler
     """
     @staticmethod
-    def _get_now():
-        return datetime.now().strftime('%Y-%m-%d %H:%M:%S,%f')[:-3]
-
-    @classmethod
-    def _get_models_list(cls, modules):
+    def _get_models_list(modules):
         models = list()
-        now = cls._get_now()
         for module in modules:
-            try:
-                importlib.import_module(module)
-                models.append(sys.modules[module])
-            except Exception as e:
-                print(f'[{now}] [{"ERROR":7}🚨] [ * {e}]')
-                continue
+            importlib.import_module(module)
+            models.append(sys.modules[module])
         return models
 
     @staticmethod
@@ -40,7 +31,7 @@ class DBPacemaker:
         return True
 
     @classmethod
-    def _get_all_tables_by_db(cls, db_binds, models_list):
+    def _get_all_tables(cls, db_binds, models_list):
         db_tables = dict()
         for models in models_list:
             for name, obj in inspect.getmembers(models):
@@ -57,44 +48,36 @@ class DBPacemaker:
 
     @classmethod
     def _get_random_tables(cls, db_binds, models_list):
-        tables_by_db = cls._get_all_tables_by_db(db_binds, models_list)
-        if not tables_by_db:
+        all_tables = cls._get_all_tables(db_binds, models_list)
+        if not all_tables:
             return None
-        return {db: cls._random_pick(tables) for db, tables in tables_by_db.items()}
+        return {db_name: cls._random_pick(tables) for db_name, tables in all_tables.items()}
 
-    @classmethod
-    def _poke(cls, db, table, display):
-        table.query.first()
+    @staticmethod
+    def _poke(db, table, db_name, display):
+        try:
+            table.query.first()
+        except:
+            db.session.rollback()
+        db.session.close()
         if display is False:
             return
-        now = cls._get_now()
-        print(f'[{now}] [{"POKE":7}👻] [ * {db.upper()} {table.__name__}]')
+        now = datetime.now().strftime('%Y-%m-%d %H:%M:%S,%f')[:-3]
+        print(f'[{now}] [{"POKE":7}👻] [ * {db_name.upper()} {table.__name__}]')
 
     @classmethod
-    def awake(cls, config, display):
+    def awake(cls, db, config, display):
         """
         Proceed every db which is available and random query a table to keep connection.
         對每個資料庫，做一次亂數取表單，透過請求表單第一個物件，保持db連線狀態。
 
-        :params config: flask config
+        :param config: flask config
         :type config: <class 'config.Config'>
-
-        :params display: to display activities of every poke
-        :type display: bool
         """
-        now = cls._get_now()
         db_binds = cls._get_db_binds(config)
-        models_path_list = getattr(config, 'MODELS_PATH_LIST', None)
-        if not models_path_list:
-            raise Exception(f'[ * = DBPacemaker = "MODELS_PATH_LIST" is empty!]')
-        models_list = cls._get_models_list(models_path_list)
-        if not models_list:
-            raise Exception(f'[ * = DBPacemaker = No available models!]')
-        random_tables = cls._get_random_tables(db_binds, models_list)
-        if not random_tables:
-            raise Exception(f'[ * = DBPacemaker = cannot source any tables!]')
-        for db, table in random_tables.items():
-            cls._poke(db=db, table=table, display=display)
+        models_list = cls._get_models_list(getattr(config, 'MODELS_PATH_LIST', list()))
+        for db_name, table in cls._get_random_tables(db_binds, models_list).items():
+            cls._poke(db=db, table=table, db_name=db_name, display=display)
 
     @staticmethod
     def _get_path():
@@ -118,21 +101,21 @@ class DBPacemaker:
             scheduler.start()
 
     @classmethod
-    def run(cls, app, config, display=False, scheduler=None):
+    def run(cls, app, db, config, display=True, scheduler=None):
         """
         Check scheduler for launching and appending awake task.
         檢查載入scheduler，插入喚醒db任務。
 
-        :params app: flask app
+        :param app: flask app
         :type app: <class 'flask.app.Flask'>
 
-        :params config: flask config
+        :param config: flask config
         :type config: <class 'config.Config'>
 
-        :params display: to display activities of every poke
+        :param display: to display activities of every poke
         :type display: bool
 
-        :params scheduler: flask_apscheduler
+        :param scheduler: flask_apscheduler
         :type scheduler: <class 'flask_apscheduler.scheduler.APScheduler'>
         """
         switch = getattr(config, 'DB_PACEMAKER_SWITCH', True)
@@ -149,7 +132,7 @@ class DBPacemaker:
             task = {
                 'id': 'keep_db_connection',
                 'func': f'{cls._get_path()}:DBPacemaker.awake',
-                'kwargs': {'config': config, 'display': display},
+                'kwargs': {'db': db, 'config': config, 'display': display},
                 'trigger': 'interval',
                 'seconds': interval
             }
